package edu.lawrence.androiddirectoryonline

import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory

class DirectoryModel : ViewModel() {
    private lateinit var restInterface : DirectoryApiService
    val key = mutableStateOf("")
    val people = mutableStateOf<List<Person>?>(null)

    init {
        val retrofit : Retrofit = Retrofit.Builder()
            .addConverterFactory(GsonConverterFactory.create())
            .baseUrl("https://cmsc106.net/directory/")
            .build()
        restInterface = retrofit.create(DirectoryApiService::class.java)
    }

    fun newUser(name : String,password : String,onSuccess : ()->Unit) {
        val newUser = User(name,password)
        viewModelScope.launch(Dispatchers.IO) {
            try {
                val result = restInterface.newUser(newUser)
                withContext(Dispatchers.Main) {
                    key.value = result
                    if(result.isNotEmpty())
                        onSuccess()
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }

    fun login(name : String,password : String,onSuccess : ()->Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            try {
                val result = restInterface.login(name,password)
                withContext(Dispatchers.Main) {
                    key.value = result
                    if(result.isNotEmpty())
                        getPeople(onSuccess = onSuccess)
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }

    fun getPeople(onSuccess : ()->Unit) {
        val keyStr = key.value
        viewModelScope.launch(Dispatchers.IO) {
            try {
                val result = restInterface.getPeople(keyStr)
                withContext(Dispatchers.Main) {
                    people.value = result
                    onSuccess()
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }

    fun addPerson(person : Person,onSuccess : ()->Unit) {
        val keyStr = key.value
        viewModelScope.launch(Dispatchers.IO) {
            try {
                restInterface.postPerson(keyStr,person)
                val result = restInterface.getPeople(keyStr)
                withContext(Dispatchers.Main) {
                    people.value = result
                    onSuccess()
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }

    fun removePerson(id : Int) {
        val keyStr = key.value
        viewModelScope.launch(Dispatchers.IO) {
            try {
                restInterface.removePerson(keyStr,id)
                val result = restInterface.getPeople(keyStr)
                withContext(Dispatchers.Main) {
                    people.value = result
                }
            } catch (e:Exception) {
                e.printStackTrace()
            }
        }
    }
}